/*
2022-05-14

update auf Arduino IDE:
2024-02-08

Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#include "rp2040x.h"
#include "pico_fmeter.h"

extern void F1_messung(int init);

extern uint8_t lcd_init();
extern void  lcd_zeile1(char *s);
extern void  lcd_zeile2(char *s);

extern int8_t   ausgabe_sperre,             // keine ser. Datenausgabe
          anzeige_funktion,                 // F+P, nur F, nur P, nur UPM
          update_LCD;                       // LCD beschriften

uint32_t *irq_tab;                          // fuer IRQ-Vektoren im RAM
uint32_t SystemCoreClock  = F_CPU;          // 125 MHz typisch


double f_ref_korrektur;                     // +/- 20 ppm per R-trimm
uint32_t ms_counter;                        // fuer freie Verwedung
volatile uint8_t ms_flag, adc_flag;         // ms-flags



void kurz_warten(void)                      // fuer kurze Verzoegerungen
{
  volatile int i = 10000;
  while(i--);
}  

// alle Millisekunden aufrufen
void TIMER_IRQ_0_IRQHandler(void)
{
  TIMER->INTR = TIMER_INTR_ALARM_0_Msk;
  TIMER->ALARM0 = TIMER->TIMELR + MS_TEILER;
  ms_counter++;
  ms_flag = 1;
  adc_flag = 1;
}

// 64 bit Timer mit 1 us Takt und loeschen
void init_timer(void)
{
//  RESETS_CLR->RESET = RESETS_RESET_timer_Msk;   // ist schon aktiv!
//  while (!(RESETS->RESET_DONE & RESETS_RESET_timer_Msk));

  NVIC_EnableIRQ(TIMER_IRQ_0_IRQn);
  NVIC_SetPriority(TIMER_IRQ_0_IRQn,3);           // geringe Prioritaet
  irq_tab[TIMER_IRQ_0_IRQn] = (uint32_t)TIMER_IRQ_0_IRQHandler; // Vektor ins RAM
  TIMER->ALARM0 = TIMER->TIMELR + MS_TEILER*100;
  TIMER->INTE |= TIMER_INTE_ALARM_0_Msk;
}

void warten_ms(uint32_t ms)
{
  do {
    while(!ms_flag);
    ms_flag = 0;
  } while(--ms);
} 


uint32_t lese_poti(void) // aktuellen Wert lesen
{
  return(analogRead(TRIMM_POTI));
}  

// auf externen Trimmer testen und ggf. zur Frequenzkorrektur verwenden
void teste_abgleich(void)
{
static int32_t summe = ADC_MAX/2 * ADC_FILTER;        // Startwert bei +/- 0
static uint32_t poti_vorhanden, poti_init;
  if(!poti_init) {
    poti_init = 1;
    IO_BANK0->GPIO26_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;  // GPIO
    PADS_BANK0->GPIO26 = PADS_BANK0_GPIO0_IE_Msk | PADS_BANK0_GPIO0_PUE_Msk;  // nur pullup = 1
    kurz_warten();
    if(lese_poti() <= 4000) poti_vorhanden = 2;       // pullup zieht nicht ganz hoch
    PADS_BANK0->GPIO26 = PADS_BANK0_GPIO0_IE_Msk | PADS_BANK0_GPIO0_PDE_Msk;  // nur pulldown = 1
    kurz_warten();
    if(lese_poti() >= 100) poti_vorhanden = 3;        // pulldown zieht nicht ganz runter
    PADS_BANK0->GPIO26 = PADS_BANK0_GPIO0_IE_Msk;     // pullup = pulldown = 0 als ADC-Eingang
  }  

  if(adc_flag && poti_vorhanden) {
    adc_flag = 0;
    summe -= summe/ADC_FILTER;
    summe += (lese_poti() - ADC_MAX/2);               // +/- 0 ppm in Mittelstellung
// Korrekturwert von +/- 2047 auf +/- 20,47 ppm von SystemCoreClock abbilden
    f_ref_korrektur = (double)SystemCoreClock * (double)summe/ADC_FILTER*1e-8;;
  }
} 

void  sende_USB(char *s){
  Serial.println(s);            // bei Bearf noch weitere Ausgabekanaele
}

void setup()
{
  // enable GPIO
  SystemCoreClock = rp2040.f_cpu();
  RESETS_CLR->RESET = RESETS_RESET_io_bank0_Msk | 
                      RESETS_RESET_pads_bank0_Msk;
  while((RESETS->RESET_DONE & (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk)) !=
        (RESETS_RESET_io_bank0_Msk | RESETS_RESET_pads_bank0_Msk));

  IO_BANK0->GPIO25_CTRL = IO_BANK0_GPIO0_CTRL_FUNCSEL_sio_0;  // LED-Ausgang
  SIO->GPIO_OE_SET = (1 << LOCAL_LED);

  irq_tab = (uint32_t *)(PPB->VTOR) + (TIMER_IRQ_0_IRQn - Reset_IRQn + 1);  // +1 wg. stack_ptr
  
  Serial.begin(DEF_BAUDRATE); // USB, Baudrate = dummy
  analogReadResolution(12);
  teste_abgleich();           // testen, ob Trimmpoti vorhanden ist

  init_timer();
  lcd_init();
  lcd_zeile1((char *)"mino Pico-FMeter");
  lcd_zeile2((char *)"V1.1  2024-02-06");
  ausgabe_sperre = 1;
  F1_messung(1);            // init
  
  while(ausgabe_sperre && TIMER->TIMELR <= 2000000) {
    F1_messung(0);          // 2 s zum Einschwingen + LCD-Meldung abwarten
  }  
  ausgabe_sperre = 0;
  update_LCD = 1;           // anzeige freigeben
}

void loop()
{
   F1_messung(0);          // run
   teste_abgleich();
// hier koennten serielle Befehle gelesen und ausgewertet werden   
}  
